@@ -1,4 +1,4 @@
-# Copyright (c) 1995-1998 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-1999 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::Image;
@@ -7,11 +7,10 @@ package Tk::Image;
 # provides a base class for them to inherit from.
 require DynaLoader;
 
-@Tk::Image::ISA = qw(DynaLoader Tk); # but are they ?
-
+use base qw(DynaLoader Tk); # but are they ?
 
 use vars qw($VERSION);
-$VERSION = '3.004'; # $Id: //depot/Tk8/Tk/Image.pm#4$
+$VERSION = '3.009'; # $Id: //depot/Tk8/Tk/Image.pm#9 $
 
 sub new
 {
@@ -67,4 +66,4 @@ sub DESTROY
 }
 
 
-1; 
+1;
