@@ -183,7 +183,7 @@ static int		TextCoords _ANSI_ARGS_((Tcl_Interp *interp,
 static void		TextDeleteChars _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, int first, int last));
 static void		TextInsert _ANSI_ARGS_((Tk_Canvas canvas,
-			    Tk_Item *itemPtr, int beforeThis, char *string));
+			    Tk_Item *itemPtr, int beforeThis, Tcl_Obj *string));
 static int		TextToArea _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, double *rectPtr));
 static double		TextToPoint _ANSI_ARGS_((Tk_Canvas canvas,
@@ -213,7 +213,7 @@ Tk_ItemType tkTextType = {
     TextToPostscript,			/* postscriptProc */
     ScaleText,				/* scaleProc */
     TranslateText,			/* translateProc */
-    (Tk_ItemIndexProc *) GetTextIndex,	/* indexProc */
+    GetTextIndex,			/* indexProc */
     SetTextCursor,			/* icursorProc */
     GetSelText,				/* selectionProc */
     TextInsert,				/* insertProc */
@@ -444,7 +444,7 @@ ConfigureText(interp, canvas, itemPtr, argc, argv, flags)
      * graphics contexts.
      */
 
-    state = itemPtr->state;
+    state = Tk_GetItemState(canvas, itemPtr);
 
     if (textPtr->activeColor != NULL ||
 	    textPtr->activeTile != NULL ||
@@ -454,10 +454,6 @@ ConfigureText(interp, canvas, itemPtr, argc, argv, flags)
 	itemPtr->redraw_flags &= ~TK_ITEM_STATE_DEPENDANT;
     }
 
-    if(state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
-    }
-
     tile = textPtr->tile;
     color = textPtr->color;
     stipple = textPtr->stipple;
@@ -668,11 +664,7 @@ ComputeTextBbox(canvas, textPtr)
 {
     Tk_CanvasTextInfo *textInfoPtr;
     int leftX, topY, width, height, fudge;
-    Tk_State state = textPtr->header.state;
-
-    if(state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
-    }
+    Tk_State state = Tk_GetItemState(canvas, &textPtr->header);
 
     Tk_FreeTextLayout(textPtr->textLayout);
     textPtr->textLayout = Tk_ComputeTextLayout(textPtr->tkfont,
@@ -782,14 +774,11 @@ DisplayCanvText(canvas, itemPtr, display, drawable, x, y, width, height)
     short drawableX, drawableY;
     Pixmap stipple;
     Tk_Tile tile;
-    Tk_State state = itemPtr->state;
+    Tk_State state = Tk_GetItemState(canvas, itemPtr);
 
     textPtr = (TextItem *) itemPtr;
     textInfoPtr = textPtr->textInfoPtr;
 
-    if(state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
-    }
     stipple = textPtr->stipple;
     tile = textPtr->tile;
     if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
@@ -955,19 +944,19 @@ DisplayCanvText(canvas, itemPtr, display, drawable, x, y, width, height)
  */
 
 static void
-TextInsert(canvas, itemPtr, beforeThis, string)
+TextInsert(canvas, itemPtr, beforeThis, ostring)
     Tk_Canvas canvas;		/* Canvas containing text item. */
     Tk_Item *itemPtr;		/* Text item to be modified. */
     int beforeThis;		/* Index of character before which text is
 				 * to be inserted. */
-    char *string;		/* New characters to be inserted. */
+    Tcl_Obj *ostring;		/* New characters to be inserted. */
 {
     TextItem *textPtr = (TextItem *) itemPtr;
-    int length = strlen(string);
+    int length = 0;
     char *new;
     Tk_CanvasTextInfo *textInfoPtr = textPtr->textInfoPtr;
 
-    /* string = Tcl_GetStringFromObj(string, &length); */
+    char * string = Tcl_GetStringFromObj(ostring, &length); 
 
     if (length == 0) {
 	return;
@@ -1124,12 +1113,9 @@ TextToPoint(canvas, itemPtr, pointPtr)
     double *pointPtr;		/* Pointer to x and y coordinates. */
 {
     TextItem *textPtr;
-    Tk_State state = itemPtr->state;
+    Tk_State state = Tk_GetItemState(canvas, itemPtr);
     double value;
 
-    if (state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
-    }
     textPtr = (TextItem *) itemPtr;
     value =  (double) Tk_DistanceToTextLayout(textPtr->textLayout,
 	    (int) pointPtr[0] - textPtr->leftEdge,
@@ -1171,11 +1157,7 @@ TextToArea(canvas, itemPtr, rectPtr)
 				 * area.  */
 {
     TextItem *textPtr;
-    Tk_State state = itemPtr->state;
-
-    if (state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
-    }
+    Tk_State state = Tk_GetItemState(canvas, itemPtr);
 
     textPtr = (TextItem *) itemPtr;
     return Tk_IntersectTextLayout(textPtr->textLayout,
@@ -1287,8 +1269,19 @@ GetTextIndex(interp, canvas, itemPtr, obj, indexPtr)
     int c;
     TkCanvas *canvasPtr = (TkCanvas *) canvas;
     Tk_CanvasTextInfo *textInfoPtr = textPtr->textInfoPtr;
-    char *string = Tcl_GetStringFromObj(obj, &length);
+    char *string;
+    int x, y;
+    double dx,dy;
+    char *end, *p;      
+    Tcl_Obj **objv;                                    
 
+    if (Tcl_ListObjGetElements(interp, obj, &c, &objv) == TCL_OK && c == 2
+	&& Tcl_GetDoubleFromObj(interp, objv[0], &dx) == TCL_OK
+	&& Tcl_GetDoubleFromObj(interp, objv[1], &dy) == TCL_OK) {
+	goto doxy;
+    } 
+
+    string = Tcl_GetStringFromObj(obj, &length);
     c = string[0];
 
     if ((c == 'e') && (strncmp(string, "end", length) == 0)) {
@@ -1310,22 +1303,19 @@ GetTextIndex(interp, canvas, itemPtr, obj, indexPtr)
 	}
 	*indexPtr = textInfoPtr->selectLast;
     } else if (c == '@') {
-	int x, y;
-	double tmp;
-	char *end, *p;
-
 	p = string+1;
-	tmp = strtod(p, &end);
+	dx = strtod(p, &end);
 	if ((end == p) || (*end != ',')) {
 	    goto badIndex;
 	}
-	x = (int) ((tmp < 0) ? tmp - 0.5 : tmp + 0.5);
 	p = end+1;
-	tmp = strtod(p, &end);
+	dy = strtod(p, &end);
 	if ((end == p) || (*end != 0)) {
 	    goto badIndex;
 	}
-	y = (int) ((tmp < 0) ? tmp - 0.5 : tmp + 0.5);
+     doxy:
+	x = (int) ((dx < 0) ? dx - 0.5 : dx + 0.5);
+	y = (int) ((dy < 0) ? dy - 0.5 : dy + 0.5);
 	*indexPtr = Tk_PointToChar(textPtr->textLayout,
 		x + canvasPtr->scrollX1 - textPtr->leftEdge,
 		y + canvasPtr->scrollY1 - textPtr->header.y1);
@@ -1472,11 +1462,8 @@ TextToPostscript(interp, canvas, itemPtr, prepass)
     char buffer[500];
     XColor *color;
     Pixmap stipple;
-    Tk_State state = itemPtr->state;
+    Tk_State state = Tk_GetItemState(canvas, itemPtr);
 
-    if(state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
-    }
     color = textPtr->color;
     stipple = textPtr->stipple;
     if (state == TK_STATE_HIDDEN || textPtr->color == NULL ||
